package com.tenlionsoft.aimz_k.model

import androidx.room.Dao
import androidx.room.Insert
import androidx.room.OnConflictStrategy
import androidx.room.Query

@Dao
interface MsgDao {
    /**
     * 更新信息状态
     */
    @Query("UPDATE db_msg SET status = :status WHERE messageId = :messageId")
    fun updateStatus(messageId: String, status: String)

    /**
     * 获取全部信息
     *
     * @return
     */
    @Query("SELECT * FROM db_msg")
    fun getAllMsg(): List<MsgBean?>?

    /**
     * 根据ID获取消息
     *
     * @param id
     * @return
     */
    @Query("SELECT * FROM db_msg WHERE id =(:id) ")
    fun getMsgById(id: String?): List<MsgBean?>?

    /**
     *
     */

    /**
     * 根据信息来源查询消息
     *
     * @param from
     * @return
     */
    @Query("SELECT * FROM db_msg WHERE `senderId`=(:from)")
    fun getMsgByFrom(from: String?): List<MsgBean?>?

    /**
     * 根据发送给谁进行查询
     *
     * @param to
     * @return
     */
    @Query("SELECT * FROM db_msg WHERE `receiverId` =(:to)")
    fun getMsgByTo(to: String?): List<MsgBean?>?


    /**
     * 根据哪里
     *
     * @param form
     * @param pageSize
     * @param page
     * @return
     */
    @Query("SELECT * FROM db_msg WHERE `senderId`=(:form) ORDER BY timestamp LIMIT (((:page)-1)*(:pageSize)),(:pageSize)")
    fun getMsgByPage(form: String?, pageSize: Int, page: Int): List<MsgBean?>?

    /**
     * 查询与单人的聊天记录
     */
    @Query("SELECT * FROM db_msg t1  WHERE t1.id IN (SELECT id FROM db_msg WHERE `senderId`=(:form) AND `receiverId`=(:to) OR `senderId`=(:to) AND `receiverId`=(:form) ORDER BY timestamp DESC LIMIT (((:page)-1)*(:pageSize)),(:pageSize)) ORDER by t1.timestamp ASC")
    fun getMsgByFromOrToPage(
        form: String?,
        to: String?,
        pageSize: Int,
        page: Int
    ): List<MsgBean?>?

    /**
     * 清空与某个人的聊天记录
     */
    @Query("DELETE FROM db_msg WHERE `senderId`=(:from) AND `receiverId`=(:to) OR `receiverId`=(:to) AND `senderId`=(:from)")
    fun delChatHistory(from: String?, to: String?)

    /**
     * 添加消息多个
     *
     * @param beans
     */
    @Insert(onConflict = OnConflictStrategy.REPLACE)
    fun insertAll(vararg beans: MsgBean)

    /**
     * 批量添加
     */
    @Insert(onConflict = OnConflictStrategy.REPLACE)
    fun insertAll(beans: List<MsgBean>)


    /**
     * 添加消息-单个
     *
     * @param bean
     */
    @Insert(onConflict = OnConflictStrategy.REPLACE)
    fun insertMsg(bean: MsgBean)

    /**
     * 清除全部聊天记录
     */
    @Query("DELETE FROM db_msg")
    fun delAllMsg()
}