package com.tenlionsoft.aimz_k.viewmodel

import android.content.Intent
import android.util.Log
import android.view.View
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.atwa.filepicker.result.FileMeta
import com.atwa.filepicker.result.ImageMeta
import com.atwa.filepicker.result.VideoMeta
import com.google.gson.Gson
import com.tenlionsoft.aimz_k.ConvertBeanUtils
import com.tenlionsoft.aimz_k.model.BodyContent
import com.tenlionsoft.aimz_k.model.CoverSealedBean
import com.tenlionsoft.aimz_k.model.DbManager
import com.tenlionsoft.aimz_k.model.MsgConvertBean
import com.tenlionsoft.aimz_k.model.PickerType
import com.tenlionsoft.aimz_k.model.Receiver
import com.tenlionsoft.aimz_k.model.Sender
import com.tenlionsoft.baselib.base.BaseViewModel
import com.tenlionsoft.baselib.contacts.ProjectConfig
import com.tenlionsoft.baselib.utils.SpUtils
import com.tenlionsoft.baselib.utils.TimeUtils
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

class ChatPageViewModel : BaseViewModel() {
    val txtMsg = MutableLiveData<String>("")
    val showSendBtn = MutableLiveData(false)//显示/隐藏发送按钮
    val showEmojiLayout = MutableLiveData(false)//显示/隐藏emoji
    val showReplyLayout = MutableLiveData(false)//显示/隐藏快速回复
    val showChooseLayout = MutableLiveData(false)//显示/隐藏选择
    val chooseType = MutableLiveData<PickerType>()//选择文件类型
    private val mGson: Gson = Gson()

    init {
        Log.e("ChatPageViewModel", "Init: ${showSendBtn.value}")
    }

    fun onTxtChange(s: CharSequence, start: Int, before: Int, count: Int) {
        this.txtMsg.value = s.toString()

        showSendBtn.value = s.isNotEmpty()
        Log.e("ChatPageViewModel", "onTxtChange: ${showSendBtn.value}")
    }

    fun onShowReplyLayout() {
        Log.e("ChatPageViewModel", "onShowReplyLayout切换前: ${showReplyLayout.value}")
        showReplyLayout.value = !showReplyLayout.value!!
        Log.e("ChatPageViewModel", "onShowReplyLayout切换后: ${showReplyLayout.value}")
        showEmojiLayout.value = false
        showChooseLayout.value = false
        showSoftKeyboard.value = !showReplyLayout.value!!
        Log.e("ChatPageViewModel", "onShowReplyLayout: ${showSoftKeyboard.value}")
    }

    /**
     * 切换emoji layout
     */
    fun onShowEmojiLayout() {
        showEmojiLayout.value = !showEmojiLayout.value!!
        showChooseLayout.value = false
        showReplyLayout.value = false
        showSoftKeyboard.value = !showEmojiLayout.value!!
    }


    /**
     * 切换选择layout
     */
    fun onShowChooseLayout() {
        showChooseLayout.value = !showChooseLayout.value!!
        showEmojiLayout.value = false
        showReplyLayout.value = false
        showSoftKeyboard.value = !showChooseLayout.value!!
    }

    /**
     * 去选择图片
     */
    fun showChoosePic() {
        chooseType.value = PickerType.TYPE_PIC
    }

    /**
     * 去选择视频
     */
    fun showChooseVideo() {
        chooseType.value = PickerType.TYPE_VIDEO
    }

    /**
     * 去选择文件
     */
    fun showChooseFile() {
        chooseType.value = PickerType.TYPE_FILE
    }

    /**
     * 上传图片
     */
    fun uploadImages(list: List<ImageMeta?>) {

    }

    /**
     * 上传视频
     */
    fun uploadVideo(meta: VideoMeta) {

    }

    /**
     * 上传文件
     */
    fun uploadFiles(meta: FileMeta) {

    }

    /**
     * 发送文本信息
     */
    fun sendTxtMsg(v: View) {
        if (txtMsg.value!!.isNotEmpty()) {
            val intent = Intent(ProjectConfig.A_S_MSG_SEND)
            val b = buildSendBean()

            viewModelScope.launch {
                withContext(Dispatchers.IO) {
                    val dao = DbManager.db.msgDao()
                    val msgBean = ConvertBeanUtils.convertBeanToMsgBean(b)
                    dao.insertMsg(msgBean)
                }
            }
            Log.e("ChatPageViewModel", "sendTxtMsg: ${b}")
            intent.putExtra("msgBean", b)
            v.context.sendBroadcast(intent)
            txtMsg.value = ""
        }

        //TODO 刷新列表
    }

    //收到消息
    fun receiveMsg(msg: String?) {
        if (!msg.isNullOrEmpty()) {
            val bean = ConvertBeanUtils.covertBean(msg)
            Log.e("ChatPageViewModel", "receiveMsg: ${bean}")
            when (bean) {
                //状态消息
                is CoverSealedBean.StateBean -> {
                    viewModelScope.launch {
                        withContext(Dispatchers.IO) {
                            val dao = DbManager.db.msgDao()
                            //TODO 并且更新Category表
                            dao.updateStatus(
                                bean.data.messageId,
                                mGson.fromJson(bean.data.body, BodyContent::class.java).statusType!!
                            )
                        }
                        //TODO 刷新列表
                    }
                }
                //消息
                is CoverSealedBean.Msg -> {
                    //插入信息表
                    //并更新category表
                    viewModelScope.launch {
                        withContext(Dispatchers.IO) {
                            val msgDao = DbManager.db.msgDao()
                            val cDao = DbManager.db.categoryDao()
                            msgDao.insertMsg(bean.msgBean)
                            cDao.updateOrInsert(bean.msgBean.senderId!!, bean.msgBean)
                        }
                    }
                    //刷新列表

                    Log.e("ChatPageViewModel", "receiveMsg接收到消息: ${bean.msgBean}")
                }
            }
        }
    }


    private fun buildSendBean(): MsgConvertBean {
        val bodyBean = BodyContent(content = txtMsg.value, null, null)
        var body = mGson.toJson(bodyBean)
        return MsgConvertBean(
            body = body,
            customMessageType = "",
            messageId = TimeUtils.getNowDateMillis().toString(),
            messageType = ProjectConfig.MSG_TEXT,
            metadata = "",
            timestamp = TimeUtils.getNowDateMillis(),
            sender = Sender(
                senderId = SpUtils.getId(),
                senderType = ""
            ),
            receiver = Receiver(
                receiverId = "2",
                receiverType = "SINGLE_USER"
            ),
            status = ProjectConfig.MSG_SEND_ING,
        )
    }
}