package com.tenlionsoft.aimz_k.viewmodel

import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.tenlionsoft.aimz_k.ConvertBeanUtils
import com.tenlionsoft.aimz_k.adapter.CategoryAdapter
import com.tenlionsoft.aimz_k.model.CoverSealedBean
import com.tenlionsoft.aimz_k.model.DbManager
import com.tenlionsoft.aimz_k.model.MsgCategoryBean
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

class MsgViewModel : ViewModel() {
    private val _pwdList = MutableLiveData<List<MsgCategoryBean>>()
    var adapter: CategoryAdapter = CategoryAdapter(_pwdList.value ?: emptyList(), this)
    var onItemClickListener: OnItemClickListener? = null
    val isLogining = MutableLiveData<Boolean>(false)
    private var mCurrentPage: Int = 1
    var isHasMore: MutableLiveData<Boolean> = MutableLiveData<Boolean>(false)

    init {
        getList(true)
    }


    /**
     * 获取列表数据
     */
    private fun getList(isFirst: Boolean) {
        viewModelScope.launch {
            if (isFirst) {
                _pwdList.value = emptyList()
                mCurrentPage = 1
            } else {
                ++mCurrentPage
            }
            val list = getCategoryListByPage(mCurrentPage)
            if (list.isNotEmpty()) {
                isHasMore.value = false
            } else {
                isHasMore.value = true
            }
            val updateData: List<MsgCategoryBean> = _pwdList.value!! + list
            _pwdList.value = updateData
            adapter.setData(_pwdList.value!!)
        }
    }

    private suspend fun getCategoryListByPage(page: Int): List<MsgCategoryBean> {
        var tempList: List<MsgCategoryBean>
        withContext(Dispatchers.IO) {
            val cDao = DbManager.db.categoryDao()
            tempList = cDao.getCategoryPage(10, page)
        }
        return tempList
    }

    /**
     * 加载更多
     */
    fun loadMore() {
        getList(false)
    }

    /**
     * 登陆websocket
     */
    fun doLoginWebsocket() {
        isLogining.value = true
    }

    //接收到信息
    fun fromMsg(msg: String) {
        val bean = ConvertBeanUtils.covertBean(msg)
        when (bean) {
            is CoverSealedBean.StateBean -> {}//消息状态
            is CoverSealedBean.Msg -> {}//消息
        }
    }

    /**
     * 刷新
     */
    fun refresh() {
        getList(true)
    }

    interface OnItemClickListener {
        fun onItemClick(msgCategoryBean: MsgCategoryBean)
    }
}