package com.tenlionsoft.aimz_k.widget

import android.view.View
import android.widget.ImageView
import androidx.databinding.BindingAdapter
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import androidx.recyclerview.widget.RecyclerView.Adapter
import com.bumptech.glide.Glide
import com.bumptech.glide.load.engine.DiskCacheStrategy
import com.bumptech.glide.request.RequestOptions
import com.tenlionsoft.aimz_k.R
import com.tenlionsoft.baselib.contacts.ProjectConfig


object BindingUtils {

    @BindingAdapter("setAdapter")
    @JvmStatic
    fun bindAdapterToRecyclerView(recyclerView: RecyclerView, adapter: Adapter<*>) {
        recyclerView.layoutManager = LinearLayoutManager(recyclerView.context)
        recyclerView.adapter = adapter
    }

    @BindingAdapter("imageUrl")
    @JvmStatic
    fun bindImgUrl(imageView: ImageView, url: String?) {
        if (url.isNullOrEmpty()) {
            imageView.setImageResource(R.drawable.app_logo_small)
        } else {
            val requestOptions = RequestOptions()
                .error(R.drawable.app_logo_small)
                .placeholder(R.drawable.app_logo_small)
                .skipMemoryCache(false)
                .diskCacheStrategy(DiskCacheStrategy.RESOURCE)
                .centerInside()
            Glide.with(imageView.context)
                .load(url)
                .apply(requestOptions)
                .into(imageView)
        }

    }

    @BindingAdapter("imgResource")
    @JvmStatic
    fun bindImgUrl(imageView: ImageView, url: Int) {
        imageView.setImageResource(url)
    }


    //发送中
    @BindingAdapter("isShowLoading")
    @JvmStatic
    fun sending(view: View, status: String) {
        when (status) {
            ProjectConfig.MSG_SEND_SUCCESS -> view.visibility = View.GONE
            ProjectConfig.MSG_SEND_FAIL -> view.visibility = View.GONE
            ProjectConfig.MSG_SEND_ING -> view.visibility = View.VISIBLE
            else -> view.visibility = View.GONE
        }
    }

    //发送失败
    @BindingAdapter("isShowFail")
    @JvmStatic
    fun sendFail(view: View, status: String) {
        when (status) {
            ProjectConfig.MSG_SEND_SUCCESS -> view.visibility = View.GONE
            ProjectConfig.MSG_SEND_FAIL -> view.visibility = View.GONE
            ProjectConfig.MSG_SEND_ING -> view.visibility = View.VISIBLE
            else -> view.visibility = View.GONE
        }
    }
}