package com.tenlionsoft.baselib.net

import android.app.Service
import android.content.Intent
import android.os.IBinder
import com.tenlionsoft.baselib.contacts.ProjectConfig
import com.tenlionsoft.baselib.model.DownInfo
import com.tenlionsoft.baselib.utils.LogUtils
import com.tenlionsoft.baselib.utils.StorageUtils
import com.tenlionsoft.baselib.utils.ToastUtils
import java.io.File

/**
 * 更新App
 */
class DownloadApkService : Service() {
    private var downInfo: DownInfo? = null
    private var oldProgress = 0
    private var mIsSuccess = true
    override fun onCreate() {
        super.onCreate()
        downInfo = DownInfo()
    }

    override fun onBind(intent: Intent?): IBinder? {
        return null;
    }

    override fun onStartCommand(intent: Intent?, flags: Int, startId: Int): Int {
        checkNotNull(intent)
        val urlStr = intent.getStringExtra(ProjectConfig.APK_DOWNLOAD_URL)
        downInfo!!.setUrl(urlStr!!)
        val dir: File = StorageUtils.getExternalCacheCustomDirectory(this)!!
        val apkName = System.currentTimeMillis().toString() + ".apk"
        downInfo!!.setApkName(apkName)
        downInfo!!.setSavePath(dir.absolutePath)
        downLoadFile()
        return super.onStartCommand(intent, flags, startId)
    }


    private fun downLoadFile() {
        val intent = Intent()
        intent.setAction(ProjectConfig.ACTION_UPDATE_START)
        sendBroadcast(intent)
        FileDownloadUtils.getInstance().download(null,
            downInfo!!.getUrl(), downInfo!!.getSavePath(), downInfo!!.getApkName()!!,
            object : DownloadResponseHandler() {
                override fun onFinish(download_file: File?) {
                    LogUtils.e("下载成功")
                    if (mIsSuccess) {
                        //收起通知栏
                        val intent: Intent = Intent()
                        intent.setAction(ProjectConfig.ACTION_UPDATE_SUCCESS)
                        intent.putExtra("apkFile", download_file)
                        sendBroadcast(intent)
                    }
                    stopSelf()
                }

                override fun onProgress(currentBytes: Long, totalBytes: Long) {
                    val progress = ((currentBytes * 1.0 / totalBytes) * 100).toInt()
                    if (progress != oldProgress) {
                        val intent = Intent()
                        intent.setAction(ProjectConfig.ACTION_UPDATE_PROGRESS)
                        intent.putExtra("progress", progress)
                        sendBroadcast(intent)
                    }
                    oldProgress = progress
                }

                override fun onFailure(error_msg: String?) {
                    LogUtils.e("下载失败")
                    mIsSuccess = false
                    ToastUtils.error("App下载失败,请稍后重试")
                    val intent = Intent()
                    intent.setAction(ProjectConfig.ACTION_UPDATE_ERROR)
                    sendBroadcast(intent)
                    stopSelf()
                }
            })
    }

}