package com.tenlionsoft.baselib.net

import android.content.Context
import android.os.Handler
import android.os.Looper
import okhttp3.Interceptor
import okhttp3.OkHttpClient
import okhttp3.Request
import java.util.concurrent.TimeUnit

class FileDownloadUtils private constructor() {
    private var client: OkHttpClient? = null
    private var instance: FileDownloadUtils? = null

    init {
        client = OkHttpClient.Builder()
            .connectTimeout(60, TimeUnit.SECONDS)
            .readTimeout(40, TimeUnit.SECONDS)
            .build()
    }

    fun download(
        context: Context?,
        url: String,
        filedir: String,
        filename: String,
        downloadResponseHandler: DownloadResponseHandler
    ) {
        val request: Request = if (context == null) {
            Request.Builder()
                .url(url)
                .build()
        } else {
            Request.Builder()
                .url(url)
                .tag(context)
                .build()
        }

        //设置拦截器
        client!!.newBuilder()
            .addNetworkInterceptor(Interceptor { chain: Interceptor.Chain ->
                val originalResponse = chain.proceed(chain.request())
                originalResponse.newBuilder()
                    .body(ResponseProgressBody(originalResponse.body!!, downloadResponseHandler))
                    .build()
            })
            .build()
            .newCall(request)
            .enqueue(
                DownloadCallback(
                    Handler(Looper.getMainLooper()),
                    downloadResponseHandler,
                    filedir,
                    filename
                )
            )
    }


    companion object {
        @Volatile
        private var instance: FileDownloadUtils? = null;
        fun getInstance() = instance ?: synchronized(this) {
            instance ?: FileDownloadUtils().also { instance = it }
        }
    }
}