package com.tenlionsoft.baselib.widget.chat

import android.annotation.SuppressLint
import android.app.Activity
import android.content.Context
import android.content.SharedPreferences
import android.graphics.Rect
import android.text.Editable
import android.text.TextWatcher
import android.util.DisplayMetrics
import android.view.MotionEvent
import android.view.View
import android.view.ViewGroup
import android.view.inputmethod.InputMethodManager
import android.widget.Button
import android.widget.EditText
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.RelativeLayout
import com.tenlionsoft.baselib.R

class ChatUiHelper {

    private var mActivity: Activity? = null
    private var mContentLayout: LinearLayout? = null //整体界面布局
    private var mBottomLayout: RelativeLayout? = null //底部布局
    private var mEmojiLayout: LinearLayout? = null //表情布局

    //    private LinearLayout mAddLayout;//添加布局
    private var mIvSend: ImageView? = null//发送按钮
    private var mAddButton: View? = null //加号按钮
    private var mAudioButton: Button? = null //录音按钮
    private var mAudioIv: ImageView? = null //录音图片
    private var mAddLayout: LinearLayout? = null

    private var mEditText: EditText? = null
    private var mInputManager: InputMethodManager? = null
    private var mSp: SharedPreferences? = null
    val EVERY_PAGE_SIZE: Int = 21

    //绑定整体界面布局
    fun bindContentLayout(bottomLayout: LinearLayout): ChatUiHelper {
        mContentLayout = bottomLayout
        return this
    }

    fun bindSendButton(iv: ImageView): ChatUiHelper {
        this.mIvSend = iv
        return this
    }

    @SuppressLint("ClickableViewAccessibility")
    fun bindEditText(editText: EditText): ChatUiHelper {
        mEditText = editText
        mEditText!!.requestFocus()
        mEditText!!.setOnTouchListener { v: View?, event: MotionEvent ->
            if (event.action == MotionEvent.ACTION_UP && mBottomLayout!!.isShown) {
                lockContentHeight() //显示软件盘时，锁定内容高度，防止跳闪。
                hideBottomLayout(true) //隐藏表情布局，显示软件盘
                // mIvEmoji.setImageResource(R.drawable.ic_emoji);
                //软件盘显示后，释放内容高度
                mEditText!!.postDelayed(
                    Runnable { this.unlockContentHeightDelayed() },
                    200L
                )
            }
            false
        }

        mEditText!!.addTextChangedListener(object : TextWatcher {
            override fun beforeTextChanged(charSequence: CharSequence, i: Int, i1: Int, i2: Int) {
            }

            override fun onTextChanged(charSequence: CharSequence, i: Int, i1: Int, i2: Int) {
                if (mEditText!!.text.toString().trim { it <= ' ' }.isNotEmpty()) {
                    mIvSend?.visibility = View.VISIBLE
//  TODO                  mAddButton!!.visibility = View.GONE
                } else {
                    mIvSend?.visibility = View.GONE
//  TODO                  mAddButton!!.visibility = View.VISIBLE
                }
            }

            override fun afterTextChanged(editable: Editable) {
            }
        })
        return this
    }

    //绑定底部布局
    fun bindBottomLayout(bottomLayout: RelativeLayout): ChatUiHelper {
        mBottomLayout = bottomLayout
        return this
    }


    //绑定表情布局
    fun bindEmojiLayout(emojiLayout: LinearLayout?): ChatUiHelper {
        mEmojiLayout = emojiLayout;
        return this
    }

    //绑定添加布局
    fun bindAddLayout(addLayout: LinearLayout): ChatUiHelper {
        mAddLayout = addLayout
        return this
    }


    //绑定语音按钮点击事件
//    fun bindAudioBtn(audioBtn: RecordButton): ChatUiHelper {
//        mAudioButton = audioBtn
//        return this
//    }

    //绑定语音图片点击事件
    fun bindAudioIv(audioIv: ImageView): ChatUiHelper {
        mAudioIv = audioIv
        audioIv.setOnClickListener { view: View? ->
            //如果录音按钮显示
            if (mAudioButton!!.isShown) {
                hideAudioButton()
                mEditText!!.requestFocus()
                showSoftInput()
            } else {
                mEditText!!.clearFocus()
                showAudioButton()
                hideEmotionLayout()
                hideMoreLayout()
            }
        }

        // UIUtils.postTaskDelay(() -> mRvMsg.smoothMoveToPosition(mRvMsg.getAdapter().getItemCount() - 1), 50);
        return this
    }

    private fun hideAudioButton() {
        mAudioButton!!.visibility = View.GONE
        mEditText!!.visibility = View.VISIBLE
        mAudioIv!!.setImageResource(R.drawable.ic_audio)
    }


    private fun showAudioButton() {
        mAudioButton!!.visibility = View.VISIBLE
        mEditText!!.visibility = View.GONE
        mAudioIv!!.setImageResource(R.drawable.ic_keyboard)
        if (mBottomLayout!!.isShown) {
            hideBottomLayout(false)
        } else {
            hideSoftInput()
        }
    }


    //绑定表情按钮点击事件
    fun bindToEmojiButton(emojiBtn: ImageView?): ChatUiHelper {
//        mIvEmoji = emojiBtn;
//        emojiBtn.setOnClickListener(new View.OnClickListener() {
//            @Override
//            public void onClick(View v) {
//                mEditText.clearFocus();
//                if (!mEmojiLayout.isShown()) {
//                    if (mAddLayout.isShown()) {
//                        showEmotionLayout();
//                        hideMoreLayout();
//                        hideAudioButton();
//                        return;
//                    }
//                } else if (mEmojiLayout.isShown() && !mAddLayout.isShown()) {
//                    mIvEmoji.setImageResource(R.drawable.ic_emoji);
//                    if (mBottomLayout.isShown()) {
//                        lockContentHeight();//显示软件盘时，锁定内容高度，防止跳闪。
//                        hideBottomLayout(true);//隐藏表情布局，显示软件盘
//                        unlockContentHeightDelayed();//软件盘显示后，释放内容高度
//                    } else {
//                        if (isSoftInputShown()) {//同上
//                            lockContentHeight();
//                            showBottomLayout();
//                            unlockContentHeightDelayed();
//                        } else {
//                            showBottomLayout();//两者都没显示，直接显示表情布局
//                        }
//                    }
//
//
//                    return;
//                }
//                showEmotionLayout();
//                hideMoreLayout();
//                hideAudioButton();
//                if (mBottomLayout.isShown()) {
//                    lockContentHeight();//显示软件盘时，锁定内容高度，防止跳闪。
//                    hideBottomLayout(true);//隐藏表情布局，显示软件盘
//                    unlockContentHeightDelayed();//软件盘显示后，释放内容高度
//                } else {
//                    if (isSoftInputShown()) {//同上
//                        lockContentHeight();
//                        showBottomLayout();
//                        unlockContentHeightDelayed();
//                    } else {
//                        showBottomLayout();//两者都没显示，直接显示表情布局
//                    }
//                }
//            }
//        });
        return this
    }


    //绑定底部加号按钮
    fun bindToAddButton(addButton: View): ChatUiHelper {
        mAddButton = addButton
        addButton.setOnClickListener { v: View? ->
            mEditText!!.clearFocus()
            hideAudioButton()
            if (mBottomLayout!!.isShown) {
                if (mAddLayout!!.isShown) {
                    lockContentHeight() //显示软件盘时，锁定内容高度，防止跳闪。
                    hideBottomLayout(true) //隐藏表情布局，显示软件盘
                    unlockContentHeightDelayed() //软件盘显示后，释放内容高度
                } else {
                    showMoreLayout()
                    hideEmotionLayout()
                }
            } else {
                if (isSoftInputShown()) { //同上
                    hideEmotionLayout()
                    showMoreLayout()
                    lockContentHeight()
                    showBottomLayout()
                    unlockContentHeightDelayed()
                } else {
                    showMoreLayout()
                    hideEmotionLayout()
                    showBottomLayout() //两者都没显示，直接显示表情布局
                }
            }
        }
        return this
    }


    private fun hideMoreLayout() {
        mAddLayout!!.visibility = View.GONE
    }

    private fun showMoreLayout() {
        mAddLayout!!.visibility = View.VISIBLE
    }


    /**
     * 隐藏底部布局
     *
     * @param showSoftInput 是否显示软件盘
     */
    fun hideBottomLayout(showSoftInput: Boolean) {
        if (mBottomLayout!!.isShown) {
            mBottomLayout!!.visibility = View.GONE
            if (showSoftInput) {
                showSoftInput()
            }
        }
    }

    private fun showBottomLayout() {
        var softInputHeight = getSupportSoftInputHeight()
        if (softInputHeight == 0) {
            softInputHeight = mSp!!.getInt(SHARE_PREFERENCE_TAG, dip2Px(270))
        }
        hideSoftInput()
        mBottomLayout!!.layoutParams.height = softInputHeight
        mBottomLayout!!.visibility = View.VISIBLE
    }


    private fun showEmotionLayout() {
        mEmojiLayout?.setVisibility(View.VISIBLE);
//        mIvEmoji.setImageResource(R.drawable.ic_keyboard);
    }

    private fun hideEmotionLayout() {
        mEmojiLayout?.setVisibility(View.GONE);
//        mIvEmoji.setImageResource(R.drawable.ic_emoji);
    }

    /**
     * 是否显示软件盘
     *
     * @return
     */
    fun isSoftInputShown(): Boolean {
        return getSupportSoftInputHeight() != 0
    }

    fun dip2Px(dip: Int): Int {
        val density = mActivity!!.applicationContext.resources.displayMetrics.density
        val px = (dip * density + 0.5f).toInt()
        return px
    }


    /**
     * 隐藏软件盘
     */
    fun hideSoftInput() {
        mInputManager!!.hideSoftInputFromWindow(mEditText!!.windowToken, 0)
    }


    /**
     * 获取软件盘的高度
     *
     * @return
     */
    private fun getSupportSoftInputHeight(): Int {
        val r = Rect()
        mActivity!!.window.decorView.getWindowVisibleDisplayFrame(r)
        //获取屏幕的高度
        val screenHeight = mActivity!!.window.decorView.rootView.height
        //计算软件盘的高度
        var softInputHeight = screenHeight - r.bottom

        if (isNavigationBarExist(mActivity!!)) {
            softInputHeight = softInputHeight - getNavigationHeight(mActivity)
        }
        //存一份到本地
        if (softInputHeight > 0) {
            mSp!!.edit().putInt(SHARE_PREFERENCE_TAG, softInputHeight).apply()
        }
        return softInputHeight
    }


    fun showSoftInput() {
        mEditText!!.requestFocus()
        mEditText!!.post { mInputManager!!.showSoftInput(mEditText, 0) }
    }

    /**
     * 锁定内容高度，防止跳闪
     */
    private fun lockContentHeight() {
        val params = mContentLayout!!.layoutParams as LinearLayout.LayoutParams
        params.height = mContentLayout!!.height
        params.weight = 0.0f
    }

    /**
     * 释放被锁定的内容高度
     */
    private fun unlockContentHeightDelayed() {
        mEditText!!.postDelayed({
            (mContentLayout!!.layoutParams as LinearLayout.LayoutParams).weight =
                1.0f
        }, 200L)
    }


    private fun getSoftButtonsBarHeight(): Int {
        val metrics = DisplayMetrics()
        mActivity!!.windowManager.defaultDisplay.getMetrics(metrics)
        val usableHeight = metrics.heightPixels
        mActivity!!.windowManager.defaultDisplay.getRealMetrics(metrics)
        val realHeight = metrics.heightPixels
        return if (realHeight > usableHeight) {
            realHeight - usableHeight
        } else {
            0
        }
    }


    val NAVIGATION: String = "navigationBarBackground"

    // 该方法需要在View完全被绘制出来之后调用，否则判断不了
    //在比如 onWindowFocusChanged（）方法中可以得到正确的结果
    fun isNavigationBarExist(activity: Activity): Boolean {
        val vp = activity.window.decorView as ViewGroup
        if (vp != null) {
            for (i in 0 until vp.childCount) {
                vp.getChildAt(i).context.packageName
                if (vp.getChildAt(i).id != View.NO_ID && NAVIGATION == activity.resources.getResourceEntryName(
                        vp.getChildAt(i).id
                    )
                ) {
                    return true
                }
            }
        }
        return false
    }


    @SuppressLint("InternalInsetResource")
    fun getNavigationHeight(activity: Context?): Int {
        if (activity == null) {
            return 0
        }
        val resources = activity.resources
        val resourceId = resources.getIdentifier(
            "navigation_bar_height",
            "dimen", "android"
        )
        var height = 0
        if (resourceId > 0) {
            //获取NavigationBar的高度
            height = resources.getDimensionPixelSize(resourceId)
        }
        return height
    }


    companion object {
        const val SHARE_PREFERENCE_NAME: String = "com.chat.ui"
        const val SHARE_PREFERENCE_TAG: String = "soft_input_height"
        fun with(activity: Activity): ChatUiHelper {
            val mChatUiHelper = ChatUiHelper()
            mChatUiHelper.mActivity = activity
            mChatUiHelper.mInputManager =
                activity.getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager
            mChatUiHelper.mSp = activity.getSharedPreferences(
                ChatUiHelper.SHARE_PREFERENCE_NAME,
                Context.MODE_PRIVATE
            )
            return mChatUiHelper
        }
    }
}